/************************************************************************
 Ty4a IRC Bot (c) 2001, 2002, ChatNet.Org.Ru

 Based on Dancer originally written by Bjorn Reese and Daniel Stenberg
 See README file for more information.
    
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 
 $Id: trio.h,v 1.1.1.1 2002/11/24 19:54:53 tzapper Exp $

************************************************************************/

#ifndef H_TRIO
#define H_TRIO

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>

/*
 * Error codes.
 *
 * Remember to add a textual description to trio_strerror.
 */
enum {
  TRIO_EOF = 1,
  TRIO_EINVAL = 2,
  TRIO_ETOOMANY = 3,
  TRIO_EDBLREF = 4,
  TRIO_EGAP = 5,
};

/* Error macros */
#define TRIO_ERROR_CODE(x) ((-(x)) & 0x00FF)
#define TRIO_ERROR_POSITION(x) ((-(x)) >> 8)
#define TRIO_ERROR_NAME(x) trio_strerror(x)

/*
 * trio_sprintf(target, format, ...)
 * trio_snprintf(target, maxsize, format, ...)
 *
 *   Build 'target' according to 'format' and succesive
 *   arguments. This is equal to the sprintf() and
 *   snprintf() functions.
 */

int trio_printf(const char *format, ...);
int trio_vprintf(const char *format, va_list args);
int trio_fprintf(FILE *fd, const char *format, ...);
int trio_vfprintf(FILE *fd, const char *format, va_list args);
int trio_sprintf(char *buffer, const char *format, ...);
int trio_snprintf(char *buffer, size_t max, const char *format, ...);
int trio_snprintfcat(char *buffer, size_t max, const char *format, ...);
int trio_vsprintf(char *buffer, const char *format, va_list args);
int trio_vsnprintf(char *buffer, size_t bufferSize, const char *format,
		   va_list args);
char *trio_aprintf(const char *format, ...);
char *trio_vaprintf(const char *format, va_list args);

int trio_sscanf(char *buffer, const char *format, ...);

const char *trio_strerror(int);


/* Replace the <stdio.h> functions */
#define printf trio_printf
#define vprintf trio_vprintf
#define fprintf trio_fprintf
#define vfprintf trio_vfprintf
#define sprintf trio_sprintf
#define snprintf trio_snprintf
#define vsprintf trio_vsprintf
#define vsnprintf trio_vsnprintf

/* There aren't stdio functions, but we make them look similar */
#define aprintf trio_aprintf
#define vaprintf trio_vaprintf

/* strio compatible names */
#define StrScan sscanf /* FIXME: must be trio_sscanf */
#define StrFormat trio_sprintf
#define StrFormatMax trio_snprintf
#define StrFormatAlloc trio_aprintf
#define StrFormatAppendMax trio_snprintfcat

#endif /* H_TRIO */
